# updated MakeRLL function
  MakeRLL2 <- function (Lon1, Lat1, SPlon, SPlat, SPangle = 0, nxny, dxdy)
  {
      if (length(dxdy) == 1)
          dxdy <- rep(dxdy, 2)
      Lon2 <- Lon1 + (nxny[1]-1) * dxdy[1] # changed from MakeRLL
      Lat2 <- Lat1 + (nxny[2]-1) * dxdy[2] # changed from MakeRLL
      projection <- list(proj = "ob_tran", o_proj = "latlong",
          o_lat_p = -SPlat, o_lon_p = 0, lon_0 = SPlon)
      RR <- project(list(x = c(Lon1, Lon2) * pi/180, y = c(Lat1,
          Lat2) * pi/180), proj = projection, inv = TRUE)
      SW <- c(RR$x[1], RR$y[1])
      NE <- c(RR$x[2], RR$y[2])
      result <- list(projection = projection, nx = nxny[1], ny = nxny[2],
          SW = SW, NE = NE)
      class(result) <- "geodomain"
      result
  }


require("ncdf4")
require("RColorBrewer")
require("maps")
library("Rfa")

#######################################
# read lon lat of CRU observations
#######################################
lon.CRU=read.table("lon_CRU_TS_v4_02.txt", header = FALSE) #path to file with longitudes of CRU data
lat.CRU=read.table("lat_CRU_TS_v4_02.txt", header = FALSE) #path to file with latitudes of CRU data

lon.CRU.matrix=matrix(lon.CRU,length(lon.CRU),length(lat.CRU))
lat.CRU.matrix=matrix(lat.CRU,length(lon.CRU),length(lat.CRU))

lon.CRU.vector = rep(unname(unlist(lon.CRU[1,])),length(lat.CRU))
lat.CRU.vector = rep(unname(unlist(lat.CRU[1,])),length(lon.CRU))

nlonlat = length(lat.CRU)*length(lon.CRU)

###########################################
# define domains
###########################################

  # define global CRU domain - 0.5° - 55 km
  CRU <- structure(list(
  projection = list(proj = "latlong",a = 6371229, es = 0),
  nx = length(lon.CRU), ny=length(lat.CRU),
  SW = c(lon.CRU.vector[1], lat.CRU.vector[1]),
  NE = c(lon.CRU.vector[nlonlat], lat.CRU.vector[nlonlat])),
  class = "geodomain")

  # .22 CAS-CORDEX ALARO domain
  CASCORDEX22=MakeRLL2(Lon1=325.68-.22*3/4,Lat1=22.88-99*.44-.22*3/4,SPlon=256.61-180,SPlat=-43.48,nxny=c(153*2,100*2),dxdy=c(.22,.22))
  CASCORDEX22.domp=DomainPoints(CASCORDEX22)

  # CAS22 ALARO domain - 0.22° - 25 km
  CAS22.inclDavies.new=Make.domain(clonlat=c(74.64,47.82), nxny=c(349,239), dxdy=c(25000,25000))
  CAS22.new=Make.domain(clonlat=c(74.64,47.82), nxny=c(349-(2*8),239-(2*8)), dxdy=c(25000,25000))

########################################
# regrid ALARO CAS22 to CRU
########################################
# get dummy CAS22 file

dummy.CAS22 = ncvar_get(nc_open("/tas_CAS-22_CNRM-CERFACS-CNRM-CM5_historical_r1i1p1_RMIB-UGent-ALARO-0_v1_day_19770101-19771231.nc"))[,,1] # path to ALARO data
map.CAS22.CRU = regrid(as.geofield(dummy.CAS22,CAS22.new),CRU) # regrid ALARO grid to CRU with bilinear interpollation

ind.map.CAS22.CRU = which(!is.na(map.CAS22.CRU),arr.ind=T)
x1.map.CAS22.CRU = range(ind.map.CAS22.CRU[,1])[1]
x2.map.CAS22.CRU = range(ind.map.CAS22.CRU[,1])[2]
y1.map.CAS22.CRU = range(ind.map.CAS22.CRU[,2])[1]
y2.map.CAS22.CRU = range(ind.map.CAS22.CRU[,2])[2]

nx.map.CAS22.CRU = (x2.map.CAS22.CRU-x1.map.CAS22.CRU)+1
ny.map.CAS22.CRU = (y2.map.CAS22.CRU-y1.map.CAS22.CRU)+1

map.CAS22.CRU.sub = subgrid(CRU,x1.map.CAS22.CRU,x2.map.CAS22.CRU,y1.map.CAS22.CRU,y2.map.CAS22.CRU) # cuts part out of the CRU data based on ALARO data

############################################
# define common map of ALARO and CRU
###########################################
dummy.CRU = ncvar_get(nc_open("cru402m_c167_201712.nc"),varid="tmp") # path to CRU data

map.CAS22.CRU.mask = array(NA,dim=c(nx.map.CAS22.CRU,ny.map.CAS22.CRU))
map.CAS22.CRU.mask[which(!is.na(map.CAS22.CRU[x1.map.CAS22.CRU:x2.map.CAS22.CRU,y1.map.CAS22.CRU:y2.map.CAS22.CRU]*dummy.CRU[x1.map.CAS22.CRU:x2.map.CAS22.CRU,y1.map.CAS22.CRU:y2.map.CAS22.CRU]))] = 1

############################################
# define REMO domain
############################################
ncfile.REMO=nc_open("tas_CAS-22_ECMWF-ERAINT_evaluation_r1i1p1_GERICS-REMO2015_v1_day_19790102-19801231.nc") # path to REMO data
nclon.REMO = ncvar_get(ncfile.REMO,varid="lon")
nclat.REMO = ncvar_get(ncfile.REMO,varid="lat")

ncproj.REMO <- list(proj="ob_tran", o_proj="latlong", o_lat_p=43.48, o_lon_p=0, lon_0=(180-103.39))
SW.REMO <- project(nclon.REMO[1, 1], nclat.REMO[1, 1], proj=ncproj.REMO)*180/pi

if(!is.null(ncfile.REMO$dim$"rlon")){
nlon.REMO = ncfile.REMO$dim$"rlon"$len
nlat.REMO = ncfile.REMO$dim$"rlat"$len
} else {
nlon.REMO = ncfile.REMO$dim$"x"$len
nlat.REMO = ncfile.REMO$dim$"y"$len}

domain.REMO=Make.domain.RLL(Lon1=SW.REMO$x,Lat1=SW.REMO$y,SPlon=(180-103.39), SPlat=-43.48,nxny=c(nlon.REMO,nlat.REMO), dxdy=c(0.22,0.22))

########################################
# regrid REMO CAS22 to CRU
########################################
dummy.REMO=ncvar_get(ncfile.REMO,varid="tas")[,,1]
map.CAS22.REMO.CRU = regrid(as.geofield(dummy.REMO,domain.REMO),CRU) # regrid REMO grid to CRU with bilinear interpollation

ind.map.CAS22.REMO.CRU = which(!is.na(map.CAS22.REMO.CRU),arr.ind=T)
x1.map.CAS22.REMO.CRU = range(ind.map.CAS22.REMO.CRU[,1])[1]
x2.map.CAS22.REMO.CRU = range(ind.map.CAS22.REMO.CRU[,1])[2]
y1.map.CAS22.REMO.CRU = range(ind.map.CAS22.REMO.CRU[,2])[1]
y2.map.CAS22.REMO.CRU = range(ind.map.CAS22.REMO.CRU[,2])[2]

nx.map.CAS22.REMO.CRU = (x2.map.CAS22.REMO.CRU-x1.map.CAS22.REMO.CRU)+1
ny.map.CAS22.REMO.CRU = (y2.map.CAS22.REMO.CRU-y1.map.CAS22.REMO.CRU)+1

map.CAS22.REMO.CRU.sub = subgrid(CRU,x1.map.CAS22.REMO.CRU,x2.map.CAS22.REMO.CRU,y1.map.CAS22.REMO.CRU,y2.map.CAS22.REMO.CRU) # cuts part out of the CRU data based on REMO data

map.CAS22.REMO.CRU.mask = array(NA,dim=c(nx.map.CAS22.REMO.CRU,ny.map.CAS22.REMO.CRU))
map.CAS22.REMO.CRU.mask[which(!is.na(map.CAS22.REMO.CRU[x1.map.CAS22.REMO.CRU:x2.map.CAS22.REMO.CRU,y1.map.CAS22.REMO.CRU:y2.map.CAS22.REMO.CRU]*dummy.CRU[x1.map.CAS22.REMO.CRU:x2.map.CAS22.REMO.CRU,y1.map.CAS22.REMO.CRU:y2.map.CAS22.REMO.CRU]))] = 1

########################################
# define ERAINT domain
############################################
ncfile.ERAINT=nc_open("tas_CAS-25_ECMWF-ERAINT_1979-2017.nc") # path to ERA-Interim data
nclon.ERAINT = ncvar_get(ncfile.ERAINT,varid="longitude")
nclat.ERAINT = ncvar_get(ncfile.ERAINT,varid="latitude") #the order is going from 70 to 17

domain.ERAINT <- structure(list(
  projection = list(proj = "latlong",a = 6371229, es = 0),
  nx = dim(nclon.ERAINT), ny=dim(nclat.ERAINT),
  SW = c(range(nclon.ERAINT)[1],range(nclat.ERAINT)[1]),
  NE = c(range(nclon.ERAINT)[2],range(nclat.ERAINT)[2])),
  class = "geodomain")

############################################
# define GPCC domain
############################################
ncfile.GPCC = nc_open("GPCC_full_data_monthly_v2018_025.nc") # path to GPCC data
GPCC.pr = ncvar_get(ncfile.GPCC,varid="precip")
nclon.GPCC = GPCC.pr[1:1440,,]
nclat.GPCC = ncvar_get(ncfile.GPCC,varid="")

domain.GPCC <- structure(list(
  projection = list(proj = "latlong",a = 6371229, es = 0),
  nx = 1440, ny = 720,
  SW = c(-180,-90),
  NE = c(180,90)),
  class = "geodomain")

############################################
# define Matsuura & Willmott domain
############################################
# Dataset from ESRL NOAA
ncfile.MW=nc_open("air.mon.mean.v501.nc") # path to UDEL data
nclon.MW = ncvar_get(ncfile.MW,varid="lon")
nclat.MW = ncvar_get(ncfile.MW,varid="lat")


domain.MW <- structure(list(
  projection = list(proj = "latlong",ellps="WGS84",a = 6378137, es = 0),
  nx = dim(nclon.MW), ny=dim(nclat.MW),
  SW = c(min(nclon.MW),min(nclat.MW)),
  NE = c(max(nclon.MW),max(nclat.MW))),
  class = "geodomain")

############################################
# save mapping objects
###########################################
CASmapping = list(
  "CRU" = CRU,
  "CAS22" = CAS22.new, #ALARO
  "CAS22.REMO" = domain.REMO,
  "CAS25.ERAINT" = domain.ERAINT,
  "CAS25.GPCC" = domain.GPCC,
  "CAS25.MW" = domain.MW,
  "x1.map.CAS22.CRU" = x1.map.CAS22.CRU, # coordinates ALARO
  "x2.map.CAS22.CRU" = x2.map.CAS22.CRU,
  "y1.map.CAS22.CRU" = y1.map.CAS22.CRU,
  "y2.map.CAS22.CRU" = y2.map.CAS22.CRU,
  "map.CAS22.CRU.sub" = map.CAS22.CRU.sub, # part cut out along the lines of the ALARO domain
  "map.CAS22.CRU.mask" = map.CAS22.CRU.mask, # mask ALARO
  "x1.map.CAS22.REMO.CRU" = x1.map.CAS22.REMO.CRU, # coordinates REMO
  "x2.map.CAS22.REMO.CRU" = x2.map.CAS22.REMO.CRU,
  "y1.map.CAS22.REMO.CRU" = y1.map.CAS22.REMO.CRU,
  "y2.map.CAS22.REMO.CRU" = y2.map.CAS22.REMO.CRU,
  "map.CAS22.REMO.CRU.sub" = map.CAS22.REMO.CRU.sub, # part cut out of global CRU map along the lines of the REMO domain which is smaller than the ALARO domain
  "map.CAS22.REMO.CRU.mask"= map.CAS22.REMO.CRU.mask) # mask REMO

save(CASmapping,file="CASmapping.RData") #save outcomes to file in a directory, so it can be used later

